import json
import logging
import os
import socket
import time
import uuid
from datetime import datetime
from enum import Enum
from logging.handlers import TimedRotatingFileHandler
from xhm_config import conf
import inspect

# 禁用外部库日志
logging.getLogger("root").setLevel(logging.INFO)
logging.getLogger("elasticsearch").setLevel(logging.WARNING)
logging.getLogger("pulsar").setLevel(logging.WARNING)
logging.getLogger("peewee").setLevel(logging.WARNING)
logging.getLogger("qcloud_cos").setLevel(logging.WARNING)


class LogColors(Enum):
    HEADER = "\033[95m"
    OKBLUE = "\033[94m"
    OKCYAN = "\033[96m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"


def _basic_config() -> logging.Logger:
    """Configure basic logging settings with daily log file rotation."""
    service_name = conf.get("service_name")
    log_level = conf.get("log_level")

    # 设置日志文件路径为 logs 目录下
    logs_folder = "logs"
    os.makedirs(logs_folder, exist_ok=True)  # 如果 logs 文件夹不存在，则创建它
    hostname = socket.gethostname()  # 获取容器的 hostname
    log_filename = os.path.join(logs_folder, f'{service_name.replace(" ", "_")}_{hostname}.log')
    # 获取 root logger
    root_logger = logging.getLogger(service_name)

    # 确保 root logger 没有重复的 handlers
    if root_logger.hasHandlers():
        root_logger.handlers.clear()  # 清除所有已有的 handlers，避免重复

    # Create a TimedRotatingFileHandler that rotates the log file every midnight
    handler = TimedRotatingFileHandler(
        log_filename, when='midnight', interval=1, backupCount=7
    )
    handler.suffix = "%Y-%m-%d.log"  # Log file name will include the date (e.g., logfile-2024-11-22.log)

    class JsonFormatter(logging.Formatter):
        def format(self, record):
            # Get the log message and additional metadata as JSON
            log_message = record.getMessage()
            log_data = {
                "timestamp": int(time.time() * 1000),
                "level": record.levelname,
                "message": log_message,
                "file": record.pathname,
                "line": record.lineno,
                "service": conf.get("service_name"),
                "host": socket.gethostname(),
                "event_type": getattr(record, 'event_type', ''),
                "event_data": getattr(record, 'event_data', {}),
                "x_request_id": getattr(record, 'x_request_id', None) or str(uuid.uuid4()),
            }
            return json.dumps(log_data, ensure_ascii=False)

    handler.setFormatter(JsonFormatter())

    # 非json
    # handler.setFormatter(logging.Formatter(
    #     "[%(asctime)s - %(filename)s:%(lineno)d - %(levelname)s] %(message)s", datefmt="%Y-%m-%d %H:%M:%S"))

    # Set up logger
    root_logger.setLevel(log_level)
    root_logger.addHandler(handler)
    return root_logger


bclog = _basic_config()


def _log_message(log_func, color: LogColors, message: str, event_type: str = "", event_data: dict = None,
                 x_request_id: str = ""):
    """Helper function to log messages with color, using stacklevel to adjust the line number shown."""

    # Dynamically get the caller's information for accurate line numbers
    if event_data is None:
        event_data = {}
    caller_frame = inspect.stack()[2]
    caller_filename = caller_frame.filename
    caller_line = caller_frame.lineno
    log_level = color.value + LogColors.BOLD.value + log_func.__name__.upper() + LogColors.ENDC.value
    log_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]  # 截取到毫秒

    colored_message = color.value + LogColors.BOLD.value + message + LogColors.ENDC.value

    extra = {"event_type": event_type, "event_data": event_data, "x_request_id": x_request_id}

    # For console log:
    log_func(message, stacklevel=3, extra=extra)  # Adjusting stacklevel to get correct line number
    print(f"{log_time} | {log_level} | {caller_filename}:{caller_line} - " + colored_message)


def warning(s: str, event_type: str = "", event_data: dict = None, x_request_id: str = ""):
    """Log warning level messages."""
    _log_message(bclog.warning, LogColors.WARNING, s,
                 event_type, event_data, x_request_id)


def info(s: str, event_type: str = "", event_data: dict = None, x_request_id: str = ""):
    """Log info level messages."""
    _log_message(bclog.info, LogColors.OKGREEN, s,
                 event_type, event_data, x_request_id)


def error(s: str, event_type: str = "", event_data: dict = None, x_request_id: str = ""):
    """Log error level messages."""
    _log_message(bclog.error, LogColors.FAIL, s,
                 event_type, event_data, x_request_id)


def debug(s: str, event_type: str = "", event_data: dict = None, x_request_id: str = ""):
    """Log error level messages."""
    _log_message(bclog.debug, LogColors.OKCYAN, s,
                 event_type, event_data, x_request_id)


def critical(s: str, event_type: str = "", event_data: dict = None, x_request_id: str = ""):
    """Log error level messages."""
    _log_message(bclog.critical, LogColors.HEADER, s,
                 event_type, event_data, x_request_id)



import os
from dynaconf import Dynaconf

# 使用 try-except 判断是否安装了 Redis
try:
    from redis.connection import ConnectionPool
    from xhm_redis import BoChuRedis

    REDIS_ENABLED = True
except ImportError:
    REDIS_ENABLED = False


class CacheConfig:
    root_path: str = None

    def get_pool(self):
        if REDIS_ENABLED:
            return ConnectionPool(
                host=self.settings.xhm_config_redis.host,
                port=self.settings.xhm_config_redis.port,
                password=self.settings.xhm_config_redis.password,
                db=self.settings.xhm_config_redis.db,
                encoding="utf-8",
                decode_responses=True,
            )
        return None

    def __init__(self):
        # 加载配置
        self.settings = Dynaconf(
            settings_files=self._get_settings_files(),
            environments=True,  # 启用分层配置
            load_dotenv=True,  # 加载环境变量
            redis_enabled=False,  # 启用 Redis 后端
        )

        # 设置环境
        self.settings.setenv(os.getenv('ENV', 'default'))

        # 初始化redis
        if self.settings.cache_mode and REDIS_ENABLED:
            self._cache = BoChuRedis(connection_pool=self.get_pool())
        else:
            self._cache = None

    def _get_settings_files(self):
        # 获取项目配置文件路径
        project_settings_file, project_secrets_file = self._get_project_path()

        # 获取 SDK 配置文件路径
        sdk_settings_file = os.path.join(os.path.dirname(__file__), 'sdk_settings.toml')
        sdk_secrets_file = os.path.join(os.path.dirname(__file__), 'sdk_secrets.toml')

        # 加载顺序 sdk_settings_file>sdk_secrets_file>project_settings_file>project_secrets_file
        # 后面的文件会覆盖前面的值
        settings_files = [sdk_settings_file]
        if os.path.exists(sdk_secrets_file):
            settings_files.append(sdk_secrets_file)

        if os.path.exists(project_settings_file):
            settings_files.append(project_settings_file)
        if os.path.exists(project_secrets_file):
            settings_files.append(project_secrets_file)
        return settings_files

    def get(self, key: str):
        return self._get_width_cache_mode(key) or self.settings.get(key, None)

    def set(self, key: str, value: any):
        if self._cache:
            self._cache.set(key, value)
        else:
            raise Exception("只允许对缓存中的值进行变更，不允许对配置文件进行变更")

    def _get_width_cache_mode(self, key: str):
        # 缓存模式
        if self.settings.cache_mode == "all" or self.settings.cache_mode == "preferred" and self.settings.get(
                f"{key}_cache", False):
            return self._cache.get(key) if self._cache else None
        return None

    def setenv(self, env: str):
        return self.settings.setenv(env)

    def all(self):
        return self.settings.as_dict()

    def info(self):
        project_settings_file, project_secrets_file = self._get_project_path()
        return {
            "root_path": self.get_root_path(),
            "project_settings": project_settings_file,
            "project_secrets": project_secrets_file}

    def _get_project_path(self):
        project_settings_file = os.path.join(os.getcwd(), 'settings.toml')
        project_secrets_file = os.path.join(os.getcwd(), '.secrets.toml')
        return project_settings_file, project_secrets_file

    def get_root_path(self):
        return self.root_path if self.root_path else os.getcwd()

    def set_root_path(self, root_path: str):
        self.root_path = root_path


conf = CacheConfig()
